#!/bin/bash
#/********************************************************/
#/* <NAME>      Docker_Installation.sh                   */
#/* <DESCRIPTION> Installation of docker and its         */
#/*               dependent packages                     */
#/* <Product>   iQ-R C Intelligent Linux module          */
#/* <Type>      Information/Intelligent module           */
#/*                                                      */
#/*     Creation date   : 21/10/2020                     */
#/*     Author          :                 */
#/* COPYRIGHT (C) 2022 Lineo Solutions, Inc. */
#/* ALL RIGHTS RESERVED                                  */
#/********************************************************/

#/********************************************************/
#/ VERSION INFORMATION
#/********************************************************/
#v1.00 : First release for C-Intelligent Debian 9 (Stretch)
#v1.01 : Support for C-Intelligent Debian 10 (Buster) added
#/********************************************************/
#/********************************************************/


#/********************************************************************************/
#/ CHANGE AS PER USER CONFIGURATION
#/********************************************************************************/
DISTRIBUTION_VERSION="Buster"                           # Default distribution is Debian 10 (Buster).
#DISTRIBUTION_VERSION="Stretch"                        	# If enabled the Docker for Debian 9 (Stretch) version will be installed.

#/********************************************************************************/
#/ CAUTION::DO NOT MODIFY BELOW CODE
#/********************************************************************************/
TMP_DIR="/tmp"
CONST_PRINT_LOGO="[Docker Installation]"
CONSOLE_MSG="@missing@"
LOG_PATH="$TMP_DIR"
DOCKER_LOG_FILE="@missing@"
CMD_EXIT_CODE=1  # init to non-zero to prevent blind passes


create_logging()
{
    LINUX_EPOCH=$(date +"%s")
    DOCKER_LOG_FILE="$LOG_PATH/Docker_Instllation_Log-$LINUX_EPOCH.log"

    type > "$DOCKER_LOG_FILE"
    if [ $? -ne 0 ]; then
        echo "Could not create log file for Docker at: $DOCKER_LOG_FILE"
        exit
    fi
}

check_network()
{
    if ping -q -c 1 -W 1 google.com >/dev/null; then      
        CONSOLE_MSG="$CONST_PRINT_LOGO Network is up"
        printing_and_logging
    else
        CONSOLE_MSG="$CONST_PRINT_LOGO Network is down. Please provide network to C-Intelligent function module"
        printing_and_logging
        exit
    fi
}

printing_and_logging()
{
    echo "$CONSOLE_MSG"
    echo "$CONSOLE_MSG" >> $DOCKER_LOG_FILE
}

check_for_root_user()
{
    if [ "$EUID" -ne 0 ]; then
        echo "Run the script using sudo command"
        exit
    fi
}

check_if_cmd_exist()
{
  INPUT_CMD="$1"
  type "$INPUT_CMD" > /dev/null 2>&1 
  CMD_EXIT_CODE=$?
}

banner()
{
    CONSOLE_MSG="############### Docker Installation ###############"
    printing_and_logging

    CONSOLE_MSG="$CONST_PRINT_LOGO :: Docker_Installation log is available at: $DOCKER_LOG_FILE"
    printing_and_logging
}


last_banner()
{
    check_if_cmd_exist "docker"

    if [ $CMD_EXIT_CODE -eq 0 ]; then
        CONSOLE_MSG="$CONST_PRINT_LOGO The Docker Installation completed successfully..."
        printing_and_logging          
        CONSOLE_MSG="$CONST_PRINT_LOGO User can execute basic example by following commands"
        printing_and_logging
        CONSOLE_MSG="./DockerInstallation.sh hello_world_example"
        printing_and_logging    
        CONSOLE_MSG="./DockerInstallation.sh start_nginx_webserver_example"
        printing_and_logging  
        CONSOLE_MSG="./DockerInstallation.sh stop_nginx_webserver_example"
        printing_and_logging          
    else
        CONSOLE_MSG="$CONST_PRINT_LOGO The Docker Installation is unsuccessfully..."
        printing_and_logging
        exit
    fi
}

download_prerequisite()
{
	check_network

    CONSOLE_MSG="$CONST_PRINT_LOGO Downloading prerequiste packages for Docker Installation"
    printing_and_logging

    remove_old_docker_version

	apt-get update

	apt-get install -y apt-transport-https ca-certificates curl gnupg2 software-properties-common
	
	if [[ "$DISTRIBUTION_VERSION" == "Buster" ]]; then
	
		wget https://curl.haxx.se/ca/cacert.pem -O /usr/local/share/ca-certificates/cacert.crt
	
		update-ca-certificates
		
	fi

	curl -fsSL https://download.docker.com/linux/$(. /etc/os-release; echo "$ID")/gpg | sudo apt-key add -

	apt-key fingerprint 0EBFCD88

	echo "deb [arch=armhf] https://download.docker.com/linux/$(. /etc/os-release; echo "$ID") $(lsb_release -cs) stable" | sudo tee /etc/apt/sources.list.d/docker.list

}

docker_installation()
{
	apt-get update
	
	apt-get install -y docker-ce
    CMD_EXIT_CODE=$?
    if [ $CMD_EXIT_CODE -ne 0 ]; then
        CONSOLE_MSG="$CONST_PRINT_LOGO The Docker Installation is unsuccessfully..."
        printing_and_logging
        exit        
    fi
}

hello_world_example()
{
    CONSOLE_MSG="$CONST_PRINT_LOGO Exeucting hello-world example ..."
    printing_and_logging
 	docker run hello-world
}

Installing_nginx_webserver_example()
{    
    CONSOLE_MSG="$CONST_PRINT_LOGO Installing nginx web server ..."
    printing_and_logging
    docker run --name docker-nginx -p 80:80 -d nginx    
}
 
start_nginx_webserver_example()   
{
    docker start docker-nginx
    CMD_EXIT_CODE=$?
    if [ $CMD_EXIT_CODE -ne 0 ]; then
        Installing_nginx_webserver_example
        docker start docker-nginx
    fi
    CONSOLE_MSG="$CONST_PRINT_LOGO nginx web server started ..."
    printing_and_logging
    CONSOLE_MSG="$CONST_PRINT_LOGO Please verify typing C-Intelli Module IP ADDRESS IP along with port 80 in the web browser..."
    printing_and_logging
    CONSOLE_MSG="$CONST_PRINT_LOGO Example : http://10.42.0.115:80 ..."
    printing_and_logging
}

stop_nginx_webserver_example()
{
    docker stop docker-nginx   
    CONSOLE_MSG="$CONST_PRINT_LOGO nginx web server stopped ..."
    printing_and_logging
}

check_old_docker_exist()
{
    check_if_cmd_exist "docker"

    if [ $CMD_EXIT_CODE -eq 0 ]; then
        read -r -p "Docker is already install. Do you still want to install ? [Y/n] " input
         
        case $input in
            [yY][eE][sS]|[yY])
        echo "Reinstallation of Docker ....."        
         ;;
            [nN][oO]|[nN])
         #echo "No"
         exit
               ;;
            *)
         echo "Invalid input..."
         exit
         ;;
        esac
    fi
}

remove_old_docker_version()
{
    apt-get remove -y docker docker-engine docker.io  
    rm /etc/apt/sources.list.d/docker.list    
}

main()
{   
    check_if_cmd_exist "docker"
    if [ $CMD_EXIT_CODE -eq 0 ]; then
        #Execute docker example when one agrument is present
        if [ "$1" ]; then
            case "$1" in
             'hello_world_example')
             hello_world_example
             ;;
             'start_nginx_webserver_example')
             start_nginx_webserver_example
             ;;
             'stop_nginx_webserver_example')
             stop_nginx_webserver_example
             ;;
            esac
            exit
        fi
    fi
    create_logging
    banner
    check_for_root_user             
    check_old_docker_exist

    if [[ "$DISTRIBUTION_VERSION" == "Buster" ]]; then
	update-alternatives --set iptables /usr/sbin/iptables-legacy
	echo "iptables alternatives changed to iptables-legacy"
    fi

    download_prerequisite
    docker_installation
    last_banner        
}

main "$@"

